/****************************************************************************
** Meta object code from reading C++ file 'ActivityInfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/ActivityInfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ActivityInfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ActivityInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto ActivityInfo::qt_create_metaobjectdata<qt_meta_tag_ZN12ActivityInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ActivityInfo",
        "nameChanged",
        "",
        "sectionChanged",
        "difficultyChanged",
        "minimalDifficultyChanged",
        "maximalDifficultyChanged",
        "iconChanged",
        "authorChanged",
        "titleChanged",
        "descriptionChanged",
        "goalChanged",
        "prerequisiteChanged",
        "manualChanged",
        "creditChanged",
        "favoriteChanged",
        "enabledChanged",
        "createdInVersionChanged",
        "levelsChanged",
        "currentLevelsChanged",
        "ignoredLevelsChanged",
        "datasetsChanged",
        "getDataset",
        "Dataset*",
        "name",
        "section",
        "difficulty",
        "minimalDifficulty",
        "maximalDifficulty",
        "icon",
        "author",
        "title",
        "description",
        "goal",
        "prerequisite",
        "manual",
        "credit",
        "favorite",
        "enabled",
        "createdInVersion",
        "levels",
        "currentLevels",
        "hasConfig",
        "hasDataset",
        "acceptDataset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sectionChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'difficultyChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimalDifficultyChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximalDifficultyChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'authorChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'descriptionChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'goalChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'prerequisiteChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'manualChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'creditChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favoriteChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'createdInVersionChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'levelsChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentLevelsChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ignoredLevelsChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'datasetsChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getDataset'
        QtMocHelpers::MethodData<Dataset *(const QString &) const>(22, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QString, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'section'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'difficulty'
        QtMocHelpers::PropertyData<quint32>(26, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'minimalDifficulty'
        QtMocHelpers::PropertyData<quint32>(27, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'maximalDifficulty'
        QtMocHelpers::PropertyData<quint32>(28, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'icon'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'author'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'goal'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'prerequisite'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'manual'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'credit'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'favorite'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'createdInVersion'
        QtMocHelpers::PropertyData<int>(39, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'levels'
        QtMocHelpers::PropertyData<QStringList>(40, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'currentLevels'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'hasConfig'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'hasDataset'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'acceptDataset'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ActivityInfo, qt_meta_tag_ZN12ActivityInfoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ActivityInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ActivityInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ActivityInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ActivityInfoE_t>.metaTypes,
    nullptr
} };

void ActivityInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActivityInfo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameChanged(); break;
        case 1: _t->sectionChanged(); break;
        case 2: _t->difficultyChanged(); break;
        case 3: _t->minimalDifficultyChanged(); break;
        case 4: _t->maximalDifficultyChanged(); break;
        case 5: _t->iconChanged(); break;
        case 6: _t->authorChanged(); break;
        case 7: _t->titleChanged(); break;
        case 8: _t->descriptionChanged(); break;
        case 9: _t->goalChanged(); break;
        case 10: _t->prerequisiteChanged(); break;
        case 11: _t->manualChanged(); break;
        case 12: _t->creditChanged(); break;
        case 13: _t->favoriteChanged(); break;
        case 14: _t->enabledChanged(); break;
        case 15: _t->createdInVersionChanged(); break;
        case 16: _t->levelsChanged(); break;
        case 17: _t->currentLevelsChanged(); break;
        case 18: _t->ignoredLevelsChanged(); break;
        case 19: _t->datasetsChanged(); break;
        case 20: { Dataset* _r = _t->getDataset((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Dataset**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::nameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::sectionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::difficultyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::minimalDifficultyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::maximalDifficultyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::iconChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::authorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::titleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::descriptionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::goalChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::prerequisiteChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::manualChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::creditChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::favoriteChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::enabledChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::createdInVersionChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::levelsChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::currentLevelsChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::ignoredLevelsChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfo::*)()>(_a, &ActivityInfo::datasetsChanged, 19))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->section(); break;
        case 2: *reinterpret_cast<quint32*>(_v) = _t->difficulty(); break;
        case 3: *reinterpret_cast<quint32*>(_v) = _t->minimalDifficulty(); break;
        case 4: *reinterpret_cast<quint32*>(_v) = _t->maximalDifficulty(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->icon(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->author(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->goal(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->prerequisite(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->manual(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->credit(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->favorite(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->createdInVersion(); break;
        case 16: *reinterpret_cast<QStringList*>(_v) = _t->levels(); break;
        case 17: *reinterpret_cast<QStringList*>(_v) = _t->currentLevels(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->hasConfig(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->hasDataset(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->acceptDataset(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setSection(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setDifficulty(*reinterpret_cast<quint32*>(_v)); break;
        case 3: _t->setMinimalDifficulty(*reinterpret_cast<quint32*>(_v)); break;
        case 4: _t->setMaximalDifficulty(*reinterpret_cast<quint32*>(_v)); break;
        case 5: _t->setIcon(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setAuthor(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setDescription(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setGoal(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setPrerequisite(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setManual(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setCredit(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setFavorite(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setCreatedInVersion(*reinterpret_cast<int*>(_v)); break;
        case 16: _t->setLevels(*reinterpret_cast<QStringList*>(_v)); break;
        case 17: _t->setCurrentLevels(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ActivityInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ActivityInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ActivityInfoE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ActivityInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void ActivityInfo::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ActivityInfo::sectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ActivityInfo::difficultyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ActivityInfo::minimalDifficultyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ActivityInfo::maximalDifficultyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ActivityInfo::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ActivityInfo::authorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ActivityInfo::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ActivityInfo::descriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ActivityInfo::goalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void ActivityInfo::prerequisiteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void ActivityInfo::manualChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void ActivityInfo::creditChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void ActivityInfo::favoriteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void ActivityInfo::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void ActivityInfo::createdInVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void ActivityInfo::levelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void ActivityInfo::currentLevelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void ActivityInfo::ignoredLevelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void ActivityInfo::datasetsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}
QT_WARNING_POP
