/****************************************************************************
** Meta object code from reading C++ file 'DownloadManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/DownloadManager.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DownloadManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15DownloadManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto DownloadManager::qt_create_metaobjectdata<qt_meta_tag_ZN15DownloadManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DownloadManager",
        "error",
        "",
        "code",
        "msg",
        "downloadStarted",
        "resource",
        "downloadProgress",
        "bytesReceived",
        "bytesTotal",
        "downloadFinished",
        "allDownloadsFinished",
        "resourceRegistered",
        "voicesRegistered",
        "backgroundMusicRegistered",
        "finishDownload",
        "finishAllDownloads",
        "downloadReadyRead",
        "handleError",
        "QNetworkReply::NetworkError",
        "downloadInProgress",
        "updateResource",
        "rt",
        "QVariantMap",
        "extra",
        "downloadResource",
        "shutdown",
        "abortDownloads",
        "getVoicesResourceForLocale",
        "locale",
        "getResourcePath",
        "data",
        "getBackgroundMusicResources",
        "haveLocalResource",
        "path",
        "initializeAssets",
        "downloadIsRunning",
        "isDataRegistered",
        "areVoicesRegistered",
        "registerResource",
        "filename"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'downloadStarted'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'downloadProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::LongLong, 9 },
        }}),
        // Signal 'downloadFinished'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'allDownloadsFinished'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'resourceRegistered'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'voicesRegistered'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backgroundMusicRegistered'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'finishDownload'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'finishAllDownloads'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'downloadReadyRead'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 3 },
        }}),
        // Slot 'downloadInProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::LongLong, 9 },
        }}),
        // Slot 'updateResource'
        QtMocHelpers::SlotData<bool(int, const QVariantMap &)>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'downloadResource'
        QtMocHelpers::SlotData<bool(int, const QVariantMap &)>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'downloadResource'
        QtMocHelpers::SlotData<bool(int)>(25, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Int, 22 },
        }}),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'abortDownloads'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getVoicesResourceForLocale'
        QtMocHelpers::MethodData<QString(const QString &) const>(28, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 29 },
        }}),
        // Method 'getResourcePath'
        QtMocHelpers::MethodData<QString(int, const QVariantMap &) const>(30, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 22 }, { 0x80000000 | 23, 31 },
        }}),
        // Method 'getBackgroundMusicResources'
        QtMocHelpers::MethodData<QString() const>(32, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'haveLocalResource'
        QtMocHelpers::MethodData<bool(const QString &) const>(33, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'initializeAssets'
        QtMocHelpers::MethodData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'downloadIsRunning'
        QtMocHelpers::MethodData<bool() const>(36, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isDataRegistered'
        QtMocHelpers::MethodData<bool(const QString &) const>(37, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 31 },
        }}),
        // Method 'areVoicesRegistered'
        QtMocHelpers::MethodData<bool(const QString &) const>(38, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 29 },
        }}),
        // Method 'registerResource'
        QtMocHelpers::MethodData<bool(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DownloadManager, qt_meta_tag_ZN15DownloadManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DownloadManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DownloadManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DownloadManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DownloadManagerE_t>.metaTypes,
    nullptr
} };

void DownloadManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DownloadManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->downloadStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->downloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 3: _t->downloadFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->allDownloadsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->resourceRegistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->voicesRegistered(); break;
        case 7: _t->backgroundMusicRegistered(); break;
        case 8: _t->finishDownload(); break;
        case 9: _t->finishAllDownloads((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->downloadReadyRead(); break;
        case 11: _t->handleError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1]))); break;
        case 12: _t->downloadInProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 13: { bool _r = _t->updateResource((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->downloadResource((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->downloadResource((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->shutdown(); break;
        case 17: _t->abortDownloads(); break;
        case 18: { QString _r = _t->getVoicesResourceForLocale((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->getResourcePath((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->getBackgroundMusicResources();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->haveLocalResource((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->initializeAssets(); break;
        case 23: { bool _r = _t->downloadIsRunning();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->isDataRegistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->areVoicesRegistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->registerResource((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(int , const QString & )>(_a, &DownloadManager::error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(const QString & )>(_a, &DownloadManager::downloadStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(qint64 , qint64 )>(_a, &DownloadManager::downloadProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(int )>(_a, &DownloadManager::downloadFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(int )>(_a, &DownloadManager::allDownloadsFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)(const QString & )>(_a, &DownloadManager::resourceRegistered, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)()>(_a, &DownloadManager::voicesRegistered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadManager::*)()>(_a, &DownloadManager::backgroundMusicRegistered, 7))
            return;
    }
}

const QMetaObject *DownloadManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DownloadManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DownloadManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DownloadManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void DownloadManager::error(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DownloadManager::downloadStarted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DownloadManager::downloadProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DownloadManager::downloadFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DownloadManager::allDownloadsFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DownloadManager::resourceRegistered(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DownloadManager::voicesRegistered()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DownloadManager::backgroundMusicRegistered()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
namespace {
struct qt_meta_tag_ZN8GComprisE_t {};
} // unnamed namespace

template <> constexpr inline auto GCompris::qt_create_metaobjectdata<qt_meta_tag_ZN8GComprisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GCompris",
        "ResourceType",
        "NONE",
        "VOICES",
        "WORDSET",
        "BACKGROUND_MUSIC",
        "FULL"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ResourceType'
        QtMocHelpers::EnumData<ResourceType>(1, 1, QMC::EnumFlags{}).add({
            {    2, ResourceType::NONE },
            {    3, ResourceType::VOICES },
            {    4, ResourceType::WORDSET },
            {    5, ResourceType::BACKGROUND_MUSIC },
            {    6, ResourceType::FULL },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN8GComprisE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN8GComprisE =
    GCompris::qt_create_metaobjectdata<qt_meta_tag_ZN8GComprisE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN8GComprisE =
    qt_staticMetaObjectContent_ZN8GComprisE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN8GComprisE =
    qt_staticMetaObjectContent_ZN8GComprisE.relocatingData;

Q_CONSTINIT const QMetaObject GCompris::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN8GComprisE.stringdata,
    qt_staticMetaObjectStaticContent_ZN8GComprisE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN8GComprisE.metaTypes,
    nullptr
} };

QT_WARNING_POP
