/****************************************************************************
** Meta object code from reading C++ file 'generator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/synth/generator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'generator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9GeneratorE_t {};
} // unnamed namespace

template <> constexpr inline auto Generator::qt_create_metaobjectdata<qt_meta_tag_ZN9GeneratorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Generator",
        "noteOn",
        "",
        "chan",
        "note",
        "vel",
        "noteOff",
        "setMode",
        "_mode",
        "setTimbre",
        "QList<int>&",
        "amplitudes",
        "phases",
        "setEnvelope",
        "ADSREnvelope",
        "env",
        "setModulation",
        "Modulation&",
        "modulation",
        "setPreset",
        "Preset&",
        "preset"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'noteOn'
        QtMocHelpers::SlotData<void(unsigned char, unsigned char, unsigned char)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 4 }, { QMetaType::UChar, 5 },
        }}),
        // Slot 'noteOff'
        QtMocHelpers::SlotData<void(unsigned char, unsigned char)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 4 },
        }}),
        // Slot 'setMode'
        QtMocHelpers::SlotData<void(unsigned int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 },
        }}),
        // Slot 'setTimbre'
        QtMocHelpers::SlotData<void(QVector<int> &, QVector<int> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Slot 'setEnvelope'
        QtMocHelpers::SlotData<void(const ADSREnvelope &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'setModulation'
        QtMocHelpers::SlotData<void(Modulation &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'setPreset'
        QtMocHelpers::SlotData<void(Preset &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Generator, qt_meta_tag_ZN9GeneratorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Generator::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GeneratorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GeneratorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9GeneratorE_t>.metaTypes,
    nullptr
} };

void Generator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Generator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->noteOn((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3]))); break;
        case 1: _t->noteOff((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2]))); break;
        case 2: _t->setMode((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 3: _t->setTimbre((*reinterpret_cast<std::add_pointer_t<QList<int>&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>&>>(_a[2]))); break;
        case 4: _t->setEnvelope((*reinterpret_cast<std::add_pointer_t<ADSREnvelope>>(_a[1]))); break;
        case 5: _t->setModulation((*reinterpret_cast<std::add_pointer_t<Modulation&>>(_a[1]))); break;
        case 6: _t->setPreset((*reinterpret_cast<std::add_pointer_t<Preset&>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Generator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Generator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GeneratorE_t>.strings))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int Generator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
