/****************************************************************************
** Meta object code from reading C++ file 'ClientNetworkMessages.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/ClientNetworkMessages.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ClientNetworkMessages.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21ClientNetworkMessagesE_t {};
} // unnamed namespace

template <> constexpr inline auto ClientNetworkMessages::qt_create_metaobjectdata<qt_meta_tag_ZN21ClientNetworkMessagesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ClientNetworkMessages",
        "newServers",
        "",
        "statusChanged",
        "hostChanged",
        "portChanged",
        "loginListReceived",
        "logins",
        "loginConfirmationReceived",
        "login",
        "logOk",
        "requestConnection",
        "requestDeviceId",
        "serverIp",
        "passwordRejected",
        "readFromSocket",
        "udpRead",
        "connected",
        "onErrorOccurred",
        "QAbstractSocket::SocketError",
        "socketError",
        "serverDisconnected",
        "ping",
        "teacherPortChanged",
        "connectToServer",
        "serverName",
        "sendLoginMessage",
        "newPassword",
        "sendActivityData",
        "activity",
        "QJsonObject",
        "data",
        "connectionStatus",
        "status",
        "netconst::ConnectionStatus",
        "host",
        "port"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newServers'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hostChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'portChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginListReceived'
        QtMocHelpers::SignalData<void(const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 7 },
        }}),
        // Signal 'loginConfirmationReceived'
        QtMocHelpers::SignalData<void(const QString &, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Signal 'requestConnection'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'passwordRejected'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readFromSocket'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'udpRead'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'connected'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onErrorOccurred'
        QtMocHelpers::SlotData<void(QAbstractSocket::SocketError)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'serverDisconnected'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ping'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'teacherPortChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'connectToServer'
        QtMocHelpers::MethodData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Method 'sendLoginMessage'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 27 },
        }}),
        // Method 'sendActivityData'
        QtMocHelpers::MethodData<void(const QString &, const QJsonObject &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { 0x80000000 | 30, 31 },
        }}),
        // Method 'connectionStatus'
        QtMocHelpers::MethodData<int()>(32, 2, QMC::AccessPublic, QMetaType::Int),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'status'
        QtMocHelpers::PropertyData<netconst::ConnectionStatus>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'host'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'port'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ClientNetworkMessages, qt_meta_tag_ZN21ClientNetworkMessagesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN21ClientNetworkMessagesE[] = {
    QMetaObject::SuperData::link<netconst::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject ClientNetworkMessages::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ClientNetworkMessagesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ClientNetworkMessagesE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN21ClientNetworkMessagesE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21ClientNetworkMessagesE_t>.metaTypes,
    nullptr
} };

void ClientNetworkMessages::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ClientNetworkMessages *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newServers(); break;
        case 1: _t->statusChanged(); break;
        case 2: _t->hostChanged(); break;
        case 3: _t->portChanged(); break;
        case 4: _t->loginListReceived((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->loginConfirmationReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->requestConnection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->passwordRejected(); break;
        case 8: _t->readFromSocket(); break;
        case 9: _t->udpRead(); break;
        case 10: _t->connected(); break;
        case 11: _t->onErrorOccurred((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 12: _t->serverDisconnected(); break;
        case 13: _t->ping(); break;
        case 14: _t->teacherPortChanged(); break;
        case 15: _t->connectToServer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->sendLoginMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->sendActivityData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJsonObject>>(_a[2]))); break;
        case 18: { int _r = _t->connectionStatus();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)()>(_a, &ClientNetworkMessages::newServers, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)()>(_a, &ClientNetworkMessages::statusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)()>(_a, &ClientNetworkMessages::hostChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)()>(_a, &ClientNetworkMessages::portChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)(const QStringList & )>(_a, &ClientNetworkMessages::loginListReceived, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)(const QString & , bool )>(_a, &ClientNetworkMessages::loginConfirmationReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)(const QString & , const QString & )>(_a, &ClientNetworkMessages::requestConnection, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientNetworkMessages::*)()>(_a, &ClientNetworkMessages::passwordRejected, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<netconst::ConnectionStatus*>(_v) = _t->status; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->host(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->port(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0:
            if (QtMocHelpers::setProperty(_t->status, *reinterpret_cast<netconst::ConnectionStatus*>(_v)))
                Q_EMIT _t->statusChanged();
            break;
        case 1: _t->setHost(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPort(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ClientNetworkMessages::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ClientNetworkMessages::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ClientNetworkMessagesE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ClientNetworkMessages::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ClientNetworkMessages::newServers()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ClientNetworkMessages::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ClientNetworkMessages::hostChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ClientNetworkMessages::portChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ClientNetworkMessages::loginListReceived(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ClientNetworkMessages::loginConfirmationReceived(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void ClientNetworkMessages::requestConnection(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void ClientNetworkMessages::passwordRejected()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
