/****************************************************************************
** Meta object code from reading C++ file 'ApplicationSettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/ApplicationSettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ApplicationSettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19ApplicationSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationSettings::qt_create_metaobjectdata<qt_meta_tag_ZN19ApplicationSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationSettings",
        "audioVoicesEnabledChanged",
        "",
        "audioEffectsEnabledChanged",
        "backgroundMusicEnabledChanged",
        "fullscreenChanged",
        "previousHeightChanged",
        "previousWidthChanged",
        "virtualKeyboardChanged",
        "localeChanged",
        "fontChanged",
        "fontCapitalizationChanged",
        "fontLetterSpacingChanged",
        "embeddedFontChanged",
        "automaticDownloadsEnabledChanged",
        "filterLevelMinChanged",
        "filterLevelMaxChanged",
        "filterLevelOverridedByCommandLineOptionChanged",
        "kioskModeChanged",
        "homeButtonVisibleChanged",
        "sectionVisibleChanged",
        "exitConfirmationChanged",
        "baseFontSizeChanged",
        "filteredBackgroundMusicChanged",
        "backgroundMusicVolumeChanged",
        "audioEffectsVolumeChanged",
        "downloadServerUrlChanged",
        "cachePathChanged",
        "userDataPathChanged",
        "teacherIdChanged",
        "teacherPortChanged",
        "exeCountChanged",
        "lastGCVersionRanChanged",
        "rendererChanged",
        "barHiddenChanged",
        "notifyAudioVoicesEnabledChanged",
        "notifyAudioEffectsEnabledChanged",
        "notifyBackgroundMusicEnabledChanged",
        "notifyFullscreenChanged",
        "notifyPreviousHeightChanged",
        "notifyPreviousWidthChanged",
        "notifyVirtualKeyboardChanged",
        "notifyLocaleChanged",
        "notifyFontChanged",
        "notifyFontCapitalizationChanged",
        "notifyFontLetterSpacingChanged",
        "notifyEmbeddedFontChanged",
        "notifyAutomaticDownloadsEnabledChanged",
        "notifyFilterLevelMinChanged",
        "notifyFilterLevelMaxChanged",
        "notifyKioskModeChanged",
        "notifyHomeButtonVisibleChanged",
        "notifySectionVisibleChanged",
        "notifyExitConfirmationChanged",
        "notifyFilteredBackgroundMusicChanged",
        "notifyBackgroundMusicVolumeChanged",
        "notifyAudioEffectsVolumeChanged",
        "notifyDownloadServerUrlChanged",
        "notifyCachePathChanged",
        "notifyUserDataPathChanged",
        "notifyExeCountChanged",
        "notifyTeacherIdChanged",
        "notifyTeacherPortChanged",
        "notifyLastGCVersionRanChanged",
        "notifyRendererChanged",
        "notifyBarHiddenChanged",
        "isFavorite",
        "activity",
        "setFavorite",
        "favorite",
        "setCurrentLevels",
        "level",
        "sync",
        "currentLevels",
        "removeLevelsSection",
        "setIgnoredLevels",
        "ignoredLevels",
        "saveBaseFontSize",
        "saveActivityConfiguration",
        "QVariantMap",
        "data",
        "loadActivityConfiguration",
        "loadActivityProgress",
        "saveActivityProgress",
        "progress",
        "useExternalWordset",
        "isAudioVoicesEnabled",
        "isAudioEffectsEnabled",
        "isBackgroundMusicEnabled",
        "isFullscreen",
        "previousHeight",
        "previousWidth",
        "isVirtualKeyboard",
        "locale",
        "font",
        "isEmbeddedFont",
        "fontCapitalization",
        "fontLetterSpacing",
        "fontLetterSpacingMin",
        "fontLetterSpacingMax",
        "isAutomaticDownloadsEnabled",
        "filterLevelMin",
        "filterLevelMax",
        "filterLevelOverridedByCommandLineOption",
        "isKioskMode",
        "isHomeButtonVisible",
        "sectionVisible",
        "exitConfirmation",
        "baseFontSize",
        "backgroundMusicVolume",
        "audioEffectsVolume",
        "baseFontSizeMin",
        "baseFontSizeMax",
        "downloadServerUrl",
        "cachePath",
        "userDataPath",
        "renderer",
        "filteredBackgroundMusic",
        "exeCount",
        "lastGCVersionRan",
        "teacherId",
        "teacherPort",
        "isBarHidden"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'audioVoicesEnabledChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'audioEffectsEnabledChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backgroundMusicEnabledChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullscreenChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previousHeightChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previousWidthChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'virtualKeyboardChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontCapitalizationChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontLetterSpacingChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'embeddedFontChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'automaticDownloadsEnabledChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filterLevelMinChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filterLevelMaxChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filterLevelOverridedByCommandLineOptionChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'kioskModeChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'homeButtonVisibleChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sectionVisibleChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exitConfirmationChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'baseFontSizeChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filteredBackgroundMusicChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backgroundMusicVolumeChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'audioEffectsVolumeChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'downloadServerUrlChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cachePathChanged'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userDataPathChanged'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'teacherIdChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'teacherPortChanged'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exeCountChanged'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastGCVersionRanChanged'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rendererChanged'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'barHiddenChanged'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'notifyAudioVoicesEnabledChanged'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyAudioEffectsEnabledChanged'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyBackgroundMusicEnabledChanged'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFullscreenChanged'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyPreviousHeightChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyPreviousWidthChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyVirtualKeyboardChanged'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyLocaleChanged'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFontChanged'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFontCapitalizationChanged'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFontLetterSpacingChanged'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyEmbeddedFontChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyAutomaticDownloadsEnabledChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFilterLevelMinChanged'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFilterLevelMaxChanged'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyKioskModeChanged'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyHomeButtonVisibleChanged'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifySectionVisibleChanged'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyExitConfirmationChanged'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyFilteredBackgroundMusicChanged'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyBackgroundMusicVolumeChanged'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyAudioEffectsVolumeChanged'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyDownloadServerUrlChanged'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyCachePathChanged'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyUserDataPathChanged'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyExeCountChanged'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyTeacherIdChanged'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyTeacherPortChanged'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyLastGCVersionRanChanged'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyRendererChanged'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyBarHiddenChanged'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'isFavorite'
        QtMocHelpers::SlotData<bool(const QString &)>(66, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'setFavorite'
        QtMocHelpers::SlotData<void(const QString &, bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::Bool, 69 },
        }}),
        // Slot 'setCurrentLevels'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, bool)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::QStringList, 71 }, { QMetaType::Bool, 72 },
        }}),
        // Slot 'setCurrentLevels'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &)>(70, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::QStringList, 71 },
        }}),
        // Slot 'currentLevels'
        QtMocHelpers::SlotData<QStringList(const QString &)>(73, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'removeLevelsSection'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setIgnoredLevels'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::QStringList, 71 }, { QMetaType::Bool, 72 },
        }}),
        // Slot 'setIgnoredLevels'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &)>(75, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::QStringList, 71 },
        }}),
        // Slot 'ignoredLevels'
        QtMocHelpers::SlotData<QStringList(const QString &)>(76, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'saveBaseFontSize'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveActivityConfiguration'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { 0x80000000 | 79, 80 },
        }}),
        // Slot 'loadActivityConfiguration'
        QtMocHelpers::SlotData<QVariantMap(const QString &)>(81, 2, QMC::AccessPublic, 0x80000000 | 79, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'loadActivityProgress'
        QtMocHelpers::SlotData<int(const QString &)>(82, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'saveActivityProgress'
        QtMocHelpers::SlotData<void(const QString &, int)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 67 }, { QMetaType::Int, 84 },
        }}),
        // Slot 'sync'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'useExternalWordset'
        QtMocHelpers::MethodData<bool()>(85, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isAudioVoicesEnabled'
        QtMocHelpers::PropertyData<bool>(86, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isAudioEffectsEnabled'
        QtMocHelpers::PropertyData<bool>(87, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isBackgroundMusicEnabled'
        QtMocHelpers::PropertyData<bool>(88, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isFullscreen'
        QtMocHelpers::PropertyData<bool>(89, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'previousHeight'
        QtMocHelpers::PropertyData<quint32>(90, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'previousWidth'
        QtMocHelpers::PropertyData<quint32>(91, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isVirtualKeyboard'
        QtMocHelpers::PropertyData<bool>(92, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'locale'
        QtMocHelpers::PropertyData<QString>(93, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'font'
        QtMocHelpers::PropertyData<QString>(94, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'isEmbeddedFont'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'fontCapitalization'
        QtMocHelpers::PropertyData<quint32>(96, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'fontLetterSpacing'
        QtMocHelpers::PropertyData<qreal>(97, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'fontLetterSpacingMin'
        QtMocHelpers::PropertyData<qreal>(98, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fontLetterSpacingMax'
        QtMocHelpers::PropertyData<qreal>(99, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isAutomaticDownloadsEnabled'
        QtMocHelpers::PropertyData<bool>(100, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'filterLevelMin'
        QtMocHelpers::PropertyData<quint32>(101, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'filterLevelMax'
        QtMocHelpers::PropertyData<quint32>(102, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'filterLevelOverridedByCommandLineOption'
        QtMocHelpers::PropertyData<bool>(103, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'isKioskMode'
        QtMocHelpers::PropertyData<bool>(104, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 16),
        // property 'isHomeButtonVisible'
        QtMocHelpers::PropertyData<bool>(105, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 17),
        // property 'sectionVisible'
        QtMocHelpers::PropertyData<bool>(106, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'exitConfirmation'
        QtMocHelpers::PropertyData<bool>(107, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'baseFontSize'
        QtMocHelpers::PropertyData<int>(108, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'backgroundMusicVolume'
        QtMocHelpers::PropertyData<qreal>(109, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'audioEffectsVolume'
        QtMocHelpers::PropertyData<qreal>(110, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'baseFontSizeMin'
        QtMocHelpers::PropertyData<int>(111, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'baseFontSizeMax'
        QtMocHelpers::PropertyData<int>(112, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'downloadServerUrl'
        QtMocHelpers::PropertyData<QString>(113, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'cachePath'
        QtMocHelpers::PropertyData<QString>(114, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'userDataPath'
        QtMocHelpers::PropertyData<QString>(115, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26),
        // property 'renderer'
        QtMocHelpers::PropertyData<QString>(116, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'filteredBackgroundMusic'
        QtMocHelpers::PropertyData<QStringList>(117, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'exeCount'
        QtMocHelpers::PropertyData<quint32>(118, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29),
        // property 'lastGCVersionRan'
        QtMocHelpers::PropertyData<int>(119, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30),
        // property 'teacherId'
        QtMocHelpers::PropertyData<QString>(120, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'teacherPort'
        QtMocHelpers::PropertyData<QString>(121, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28),
        // property 'isBarHidden'
        QtMocHelpers::PropertyData<bool>(122, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 32),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ApplicationSettings, qt_meta_tag_ZN19ApplicationSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ApplicationSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ApplicationSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ApplicationSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19ApplicationSettingsE_t>.metaTypes,
    nullptr
} };

void ApplicationSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->audioVoicesEnabledChanged(); break;
        case 1: _t->audioEffectsEnabledChanged(); break;
        case 2: _t->backgroundMusicEnabledChanged(); break;
        case 3: _t->fullscreenChanged(); break;
        case 4: _t->previousHeightChanged(); break;
        case 5: _t->previousWidthChanged(); break;
        case 6: _t->virtualKeyboardChanged(); break;
        case 7: _t->localeChanged(); break;
        case 8: _t->fontChanged(); break;
        case 9: _t->fontCapitalizationChanged(); break;
        case 10: _t->fontLetterSpacingChanged(); break;
        case 11: _t->embeddedFontChanged(); break;
        case 12: _t->automaticDownloadsEnabledChanged(); break;
        case 13: _t->filterLevelMinChanged(); break;
        case 14: _t->filterLevelMaxChanged(); break;
        case 15: _t->filterLevelOverridedByCommandLineOptionChanged(); break;
        case 16: _t->kioskModeChanged(); break;
        case 17: _t->homeButtonVisibleChanged(); break;
        case 18: _t->sectionVisibleChanged(); break;
        case 19: _t->exitConfirmationChanged(); break;
        case 20: _t->baseFontSizeChanged(); break;
        case 21: _t->filteredBackgroundMusicChanged(); break;
        case 22: _t->backgroundMusicVolumeChanged(); break;
        case 23: _t->audioEffectsVolumeChanged(); break;
        case 24: _t->downloadServerUrlChanged(); break;
        case 25: _t->cachePathChanged(); break;
        case 26: _t->userDataPathChanged(); break;
        case 27: _t->teacherIdChanged(); break;
        case 28: _t->teacherPortChanged(); break;
        case 29: _t->exeCountChanged(); break;
        case 30: _t->lastGCVersionRanChanged(); break;
        case 31: _t->rendererChanged(); break;
        case 32: _t->barHiddenChanged(); break;
        case 33: _t->notifyAudioVoicesEnabledChanged(); break;
        case 34: _t->notifyAudioEffectsEnabledChanged(); break;
        case 35: _t->notifyBackgroundMusicEnabledChanged(); break;
        case 36: _t->notifyFullscreenChanged(); break;
        case 37: _t->notifyPreviousHeightChanged(); break;
        case 38: _t->notifyPreviousWidthChanged(); break;
        case 39: _t->notifyVirtualKeyboardChanged(); break;
        case 40: _t->notifyLocaleChanged(); break;
        case 41: _t->notifyFontChanged(); break;
        case 42: _t->notifyFontCapitalizationChanged(); break;
        case 43: _t->notifyFontLetterSpacingChanged(); break;
        case 44: _t->notifyEmbeddedFontChanged(); break;
        case 45: _t->notifyAutomaticDownloadsEnabledChanged(); break;
        case 46: _t->notifyFilterLevelMinChanged(); break;
        case 47: _t->notifyFilterLevelMaxChanged(); break;
        case 48: _t->notifyKioskModeChanged(); break;
        case 49: _t->notifyHomeButtonVisibleChanged(); break;
        case 50: _t->notifySectionVisibleChanged(); break;
        case 51: _t->notifyExitConfirmationChanged(); break;
        case 52: _t->notifyFilteredBackgroundMusicChanged(); break;
        case 53: _t->notifyBackgroundMusicVolumeChanged(); break;
        case 54: _t->notifyAudioEffectsVolumeChanged(); break;
        case 55: _t->notifyDownloadServerUrlChanged(); break;
        case 56: _t->notifyCachePathChanged(); break;
        case 57: _t->notifyUserDataPathChanged(); break;
        case 58: _t->notifyExeCountChanged(); break;
        case 59: _t->notifyTeacherIdChanged(); break;
        case 60: _t->notifyTeacherPortChanged(); break;
        case 61: _t->notifyLastGCVersionRanChanged(); break;
        case 62: _t->notifyRendererChanged(); break;
        case 63: _t->notifyBarHiddenChanged(); break;
        case 64: { bool _r = _t->isFavorite((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->setFavorite((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 66: _t->setCurrentLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 67: _t->setCurrentLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 68: { QStringList _r = _t->currentLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 69: _t->removeLevelsSection(); break;
        case 70: _t->setIgnoredLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 71: _t->setIgnoredLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 72: { QStringList _r = _t->ignoredLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 73: _t->saveBaseFontSize(); break;
        case 74: _t->saveActivityConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 75: { QVariantMap _r = _t->loadActivityConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 76: { int _r = _t->loadActivityProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 77: _t->saveActivityProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 78: _t->sync(); break;
        case 79: { bool _r = _t->useExternalWordset();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::audioVoicesEnabledChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::audioEffectsEnabledChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::backgroundMusicEnabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::fullscreenChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::previousHeightChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::previousWidthChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::virtualKeyboardChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::localeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::fontChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::fontCapitalizationChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::fontLetterSpacingChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::embeddedFontChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::automaticDownloadsEnabledChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::filterLevelMinChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::filterLevelMaxChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::filterLevelOverridedByCommandLineOptionChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::kioskModeChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::homeButtonVisibleChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::sectionVisibleChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::exitConfirmationChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::baseFontSizeChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::filteredBackgroundMusicChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::backgroundMusicVolumeChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::audioEffectsVolumeChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::downloadServerUrlChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::cachePathChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::userDataPathChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::teacherIdChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::teacherPortChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::exeCountChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::lastGCVersionRanChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::rendererChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationSettings::*)()>(_a, &ApplicationSettings::barHiddenChanged, 32))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isAudioVoicesEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isAudioEffectsEnabled(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isBackgroundMusicEnabled(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isFullscreen(); break;
        case 4: *reinterpret_cast<quint32*>(_v) = _t->previousHeight(); break;
        case 5: *reinterpret_cast<quint32*>(_v) = _t->previousWidth(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isVirtualKeyboard(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->locale(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->font(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isEmbeddedFont(); break;
        case 10: *reinterpret_cast<quint32*>(_v) = _t->fontCapitalization(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->fontLetterSpacing(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->fontLetterSpacingMin(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->fontLetterSpacingMax(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isAutomaticDownloadsEnabled(); break;
        case 15: *reinterpret_cast<quint32*>(_v) = _t->filterLevelMin(); break;
        case 16: *reinterpret_cast<quint32*>(_v) = _t->filterLevelMax(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->filterLevelOverridedByCommandLineOption(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isKioskMode(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isHomeButtonVisible(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->sectionVisible(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->exitConfirmation(); break;
        case 22: *reinterpret_cast<int*>(_v) = _t->baseFontSize(); break;
        case 23: *reinterpret_cast<qreal*>(_v) = _t->backgroundMusicVolume(); break;
        case 24: *reinterpret_cast<qreal*>(_v) = _t->audioEffectsVolume(); break;
        case 25: *reinterpret_cast<int*>(_v) = _t->baseFontSizeMin(); break;
        case 26: *reinterpret_cast<int*>(_v) = _t->baseFontSizeMax(); break;
        case 27: *reinterpret_cast<QString*>(_v) = _t->downloadServerUrl(); break;
        case 28: *reinterpret_cast<QString*>(_v) = _t->cachePath(); break;
        case 29: *reinterpret_cast<QString*>(_v) = _t->userDataPath(); break;
        case 30: *reinterpret_cast<QString*>(_v) = _t->renderer(); break;
        case 31: *reinterpret_cast<QStringList*>(_v) = _t->filteredBackgroundMusic(); break;
        case 32: *reinterpret_cast<quint32*>(_v) = _t->exeCount(); break;
        case 33: *reinterpret_cast<int*>(_v) = _t->lastGCVersionRan(); break;
        case 34: *reinterpret_cast<QString*>(_v) = _t->teacherId(); break;
        case 35: *reinterpret_cast<QString*>(_v) = _t->teacherPort(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->isBarHidden(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setIsAudioVoicesEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setIsAudioEffectsEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setIsBackgroundMusicEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFullscreen(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setPreviousHeight(*reinterpret_cast<quint32*>(_v)); break;
        case 5: _t->setPreviousWidth(*reinterpret_cast<quint32*>(_v)); break;
        case 6: _t->setVirtualKeyboard(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setLocale(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setFont(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setIsEmbeddedFont(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setFontCapitalization(*reinterpret_cast<quint32*>(_v)); break;
        case 11: _t->setFontLetterSpacing(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setIsAutomaticDownloadsEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setFilterLevelMin(*reinterpret_cast<quint32*>(_v)); break;
        case 16: _t->setFilterLevelMax(*reinterpret_cast<quint32*>(_v)); break;
        case 17: _t->setFilterLevelOverridedByCommandLineOption(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setKioskMode(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setHomeButtonVisible(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setSectionVisible(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setExitConfirmation(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setBaseFontSize(*reinterpret_cast<int*>(_v)); break;
        case 23: _t->setBackgroundMusicVolume(*reinterpret_cast<qreal*>(_v)); break;
        case 24: _t->setAudioEffectsVolume(*reinterpret_cast<qreal*>(_v)); break;
        case 27: _t->setDownloadServerUrl(*reinterpret_cast<QString*>(_v)); break;
        case 28: _t->setCachePath(*reinterpret_cast<QString*>(_v)); break;
        case 29: _t->setUserDataPath(*reinterpret_cast<QString*>(_v)); break;
        case 30: _t->setRenderer(*reinterpret_cast<QString*>(_v)); break;
        case 31: _t->setFilteredBackgroundMusic(*reinterpret_cast<QStringList*>(_v)); break;
        case 32: _t->setExeCount(*reinterpret_cast<quint32*>(_v)); break;
        case 33: _t->setLastGCVersionRan(*reinterpret_cast<int*>(_v)); break;
        case 34: _t->setTeacherId(*reinterpret_cast<QString*>(_v)); break;
        case 35: _t->setTeacherPort(*reinterpret_cast<QString*>(_v)); break;
        case 36: _t->setBarHidden(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ApplicationSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ApplicationSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ApplicationSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 80)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 80;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 80)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 80;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    return _id;
}

// SIGNAL 0
void ApplicationSettings::audioVoicesEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ApplicationSettings::audioEffectsEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ApplicationSettings::backgroundMusicEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ApplicationSettings::fullscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ApplicationSettings::previousHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ApplicationSettings::previousWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ApplicationSettings::virtualKeyboardChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ApplicationSettings::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ApplicationSettings::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ApplicationSettings::fontCapitalizationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void ApplicationSettings::fontLetterSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void ApplicationSettings::embeddedFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void ApplicationSettings::automaticDownloadsEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void ApplicationSettings::filterLevelMinChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void ApplicationSettings::filterLevelMaxChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void ApplicationSettings::filterLevelOverridedByCommandLineOptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void ApplicationSettings::kioskModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void ApplicationSettings::homeButtonVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void ApplicationSettings::sectionVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void ApplicationSettings::exitConfirmationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void ApplicationSettings::baseFontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void ApplicationSettings::filteredBackgroundMusicChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void ApplicationSettings::backgroundMusicVolumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void ApplicationSettings::audioEffectsVolumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void ApplicationSettings::downloadServerUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void ApplicationSettings::cachePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void ApplicationSettings::userDataPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void ApplicationSettings::teacherIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void ApplicationSettings::teacherPortChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void ApplicationSettings::exeCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void ApplicationSettings::lastGCVersionRanChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void ApplicationSettings::rendererChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void ApplicationSettings::barHiddenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}
QT_WARNING_POP
