/****************************************************************************
** Meta object code from reading C++ file 'ActivityInfoTree.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/ActivityInfoTree.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ActivityInfoTree.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ActivityInfoTreeE_t {};
} // unnamed namespace

template <> constexpr inline auto ActivityInfoTree::qt_create_metaobjectdata<qt_meta_tag_ZN16ActivityInfoTreeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ActivityInfoTree",
        "menuTreeChanged",
        "",
        "currentActivityChanged",
        "allCharactersChanged",
        "startingActivityChanged",
        "startingLevelChanged",
        "minMaxFiltersChanged",
        "levelMin",
        "levelMax",
        "doSynchronize",
        "filterByTag",
        "tag",
        "category",
        "emitChanged",
        "resetLevels",
        "activity",
        "filterEnabledActivities",
        "filterCreatedWithinVersions",
        "firstVersion",
        "lastVersion",
        "filterBySearch",
        "text",
        "filterByDifficulty",
        "setCurrentActivityFromName",
        "name",
        "rootMenu",
        "ActivityInfo*",
        "menuTreeFull",
        "QQmlListProperty<ActivityInfo>",
        "menuTree",
        "currentActivity",
        "characters",
        "QVariantList",
        "startingActivity",
        "startingLevel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'menuTreeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentActivityChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allCharactersChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startingActivityChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startingLevelChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'minMaxFiltersChanged'
        QtMocHelpers::SlotData<void(quint32, quint32, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 }, { QMetaType::UInt, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Slot 'minMaxFiltersChanged'
        QtMocHelpers::SlotData<void(quint32, quint32)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::UInt, 8 }, { QMetaType::UInt, 9 },
        }}),
        // Slot 'filterByTag'
        QtMocHelpers::SlotData<void(const QString &, const QString &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'filterByTag'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'filterByTag'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'resetLevels'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'filterEnabledActivities'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'filterEnabledActivities'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'filterCreatedWithinVersions'
        QtMocHelpers::SlotData<void(int, int, bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'filterCreatedWithinVersions'
        QtMocHelpers::SlotData<void(int, int)>(18, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
        }}),
        // Slot 'filterBySearch'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'filterByDifficulty'
        QtMocHelpers::SlotData<void(quint32, quint32)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 8 }, { QMetaType::UInt, 9 },
        }}),
        // Slot 'setCurrentActivityFromName'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rootMenu'
        QtMocHelpers::PropertyData<ActivityInfo*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'menuTreeFull'
        QtMocHelpers::PropertyData<QQmlListProperty<ActivityInfo>>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'menuTree'
        QtMocHelpers::PropertyData<QQmlListProperty<ActivityInfo>>(30, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'currentActivity'
        QtMocHelpers::PropertyData<ActivityInfo*>(31, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'characters'
        QtMocHelpers::PropertyData<QVariantList>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'startingActivity'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'startingLevel'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ActivityInfoTree, qt_meta_tag_ZN16ActivityInfoTreeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ActivityInfoTree::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ActivityInfoTreeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ActivityInfoTreeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ActivityInfoTreeE_t>.metaTypes,
    nullptr
} };

void ActivityInfoTree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActivityInfoTree *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->menuTreeChanged(); break;
        case 1: _t->currentActivityChanged(); break;
        case 2: _t->allCharactersChanged(); break;
        case 3: _t->startingActivityChanged(); break;
        case 4: _t->startingLevelChanged(); break;
        case 5: _t->minMaxFiltersChanged((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->minMaxFiltersChanged((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2]))); break;
        case 7: _t->filterByTag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 8: _t->filterByTag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->filterByTag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->resetLevels((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->filterEnabledActivities((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->filterEnabledActivities(); break;
        case 13: _t->filterCreatedWithinVersions((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 14: _t->filterCreatedWithinVersions((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 15: _t->filterBySearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->filterByDifficulty((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2]))); break;
        case 17: _t->setCurrentActivityFromName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActivityInfoTree::*)()>(_a, &ActivityInfoTree::menuTreeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfoTree::*)()>(_a, &ActivityInfoTree::currentActivityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfoTree::*)()>(_a, &ActivityInfoTree::allCharactersChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfoTree::*)()>(_a, &ActivityInfoTree::startingActivityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityInfoTree::*)()>(_a, &ActivityInfoTree::startingLevelChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ActivityInfo* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ActivityInfo**>(_v) = _t->getRootMenu(); break;
        case 1: *reinterpret_cast<QQmlListProperty<ActivityInfo>*>(_v) = _t->getMenuTreeFull(); break;
        case 2: *reinterpret_cast<QQmlListProperty<ActivityInfo>*>(_v) = _t->menuTree(); break;
        case 3: *reinterpret_cast<ActivityInfo**>(_v) = _t->getCurrentActivity(); break;
        case 4: *reinterpret_cast<QVariantList*>(_v) = _t->allCharacters(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->m_startingActivity; break;
        case 6: *reinterpret_cast<int*>(_v) = _t->m_startingLevel; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setCurrentActivity(*reinterpret_cast<ActivityInfo**>(_v)); break;
        case 5:
            if (QtMocHelpers::setProperty(_t->m_startingActivity, *reinterpret_cast<QString*>(_v)))
                Q_EMIT _t->startingActivityChanged();
            break;
        case 6:
            if (QtMocHelpers::setProperty(_t->m_startingLevel, *reinterpret_cast<int*>(_v)))
                Q_EMIT _t->startingLevelChanged();
            break;
        default: break;
        }
    }
}

const QMetaObject *ActivityInfoTree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ActivityInfoTree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ActivityInfoTreeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ActivityInfoTree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ActivityInfoTree::menuTreeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ActivityInfoTree::currentActivityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ActivityInfoTree::allCharactersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ActivityInfoTree::startingActivityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ActivityInfoTree::startingLevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
