/****************************************************************************
** Meta object code from reading C++ file 'ApplicationInfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/ApplicationInfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ApplicationInfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15ApplicationInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationInfo::qt_create_metaobjectdata<qt_meta_tag_ZN15ApplicationInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationInfo",
        "applicationWidthChanged",
        "",
        "portraitModeChanged",
        "ratioChanged",
        "fontRatioChanged",
        "applicationSettingsChanged",
        "fullscreenChanged",
        "useSoftwareRendererChanged",
        "isBox2DInstalledChanged",
        "serverConnectionAcceptedChanged",
        "serverStatusColorChanged",
        "getResourceDataPaths",
        "getAudioFilePath",
        "file",
        "getAudioFilePathForLocale",
        "localeName",
        "getLocaleFilePath",
        "getSystemExcludedFonts",
        "getFontsFromRcc",
        "getBackgroundMusicFromRcc",
        "screenshot",
        "notifyPortraitMode",
        "notifyFullscreenChanged",
        "getVoicesLocale",
        "locale",
        "requestAudioFocus",
        "abandonAudioFocus",
        "localeCompare",
        "a",
        "b",
        "localeSort",
        "QVariantList",
        "list",
        "switchLocale",
        "setServerConnectionAccepted",
        "connectionAccepted",
        "setServerStatusColor",
        "statusColor",
        "getNativeOrientation",
        "Qt::ScreenOrientation",
        "setRequestedOrientation",
        "orientation",
        "getRequestedOrientation",
        "sensorIsSupported",
        "sensorType",
        "setKeepScreenOn",
        "value",
        "applicationWidth",
        "platform",
        "Platform",
        "isMobile",
        "hasShader",
        "isPortraitMode",
        "ratio",
        "fontRatio",
        "localeShort",
        "GCVersion",
        "GCVersionCode",
        "QTVersion",
        "QTMinorVersion",
        "OpenSSLVersion",
        "CompressedAudio",
        "isDownloadAllowed",
        "useSoftwareRenderer",
        "isBox2DInstalled",
        "serverConnectionAccepted",
        "serverStatusColor",
        "Linux",
        "Windows",
        "MacOSX",
        "Android",
        "Ios",
        "Blackberry",
        "UbuntuTouchOS"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'applicationWidthChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'portraitModeChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ratioChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontRatioChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'applicationSettingsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullscreenChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'useSoftwareRendererChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isBox2DInstalledChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverConnectionAcceptedChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverStatusColorChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getResourceDataPaths'
        QtMocHelpers::SlotData<QStringList()>(12, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getAudioFilePath'
        QtMocHelpers::SlotData<QString(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'getAudioFilePathForLocale'
        QtMocHelpers::SlotData<QString(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'getLocaleFilePath'
        QtMocHelpers::SlotData<QString(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'getSystemExcludedFonts'
        QtMocHelpers::SlotData<QStringList()>(18, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getFontsFromRcc'
        QtMocHelpers::SlotData<QStringList()>(19, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getBackgroundMusicFromRcc'
        QtMocHelpers::SlotData<QStringList()>(20, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'notifyPortraitMode'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'notifyFullscreenChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getVoicesLocale'
        QtMocHelpers::MethodData<QString(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 25 },
        }}),
        // Method 'requestAudioFocus'
        QtMocHelpers::MethodData<bool() const>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'abandonAudioFocus'
        QtMocHelpers::MethodData<void() const>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'localeCompare'
        QtMocHelpers::MethodData<int(const QString &, const QString &, const QString &) const>(28, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 29 }, { QMetaType::QString, 30 }, { QMetaType::QString, 25 },
        }}),
        // Method 'localeCompare'
        QtMocHelpers::MethodData<int(const QString &, const QString &) const>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 29 }, { QMetaType::QString, 30 },
        }}),
        // Method 'localeSort'
        QtMocHelpers::MethodData<QVariantList(QVariantList, const QString &) const>(31, 2, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QString, 25 },
        }}),
        // Method 'localeSort'
        QtMocHelpers::MethodData<QVariantList(QVariantList) const>(31, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 32, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'switchLocale'
        QtMocHelpers::MethodData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Method 'switchLocale'
        QtMocHelpers::MethodData<void()>(34, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'setServerConnectionAccepted'
        QtMocHelpers::MethodData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 36 },
        }}),
        // Method 'setServerStatusColor'
        QtMocHelpers::MethodData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Method 'getNativeOrientation'
        QtMocHelpers::MethodData<Qt::ScreenOrientation()>(39, 2, QMC::AccessPublic, 0x80000000 | 40),
        // Method 'setRequestedOrientation'
        QtMocHelpers::MethodData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
        // Method 'getRequestedOrientation'
        QtMocHelpers::MethodData<int()>(43, 2, QMC::AccessPublic, QMetaType::Int),
        // Method 'sensorIsSupported'
        QtMocHelpers::MethodData<bool(const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 45 },
        }}),
        // Method 'setKeepScreenOn'
        QtMocHelpers::MethodData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'applicationWidth'
        QtMocHelpers::PropertyData<int>(48, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'platform'
        QtMocHelpers::PropertyData<enum Platform>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'isMobile'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'hasShader'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isPortraitMode'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'ratio'
        QtMocHelpers::PropertyData<qreal>(54, QMetaType::QReal, QMC::DefaultPropertyFlags, 2),
        // property 'fontRatio'
        QtMocHelpers::PropertyData<qreal>(55, QMetaType::QReal, QMC::DefaultPropertyFlags, 3),
        // property 'localeShort'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'GCVersion'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'GCVersionCode'
        QtMocHelpers::PropertyData<int>(58, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'QTVersion'
        QtMocHelpers::PropertyData<QString>(59, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'QTMinorVersion'
        QtMocHelpers::PropertyData<int>(60, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'OpenSSLVersion'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'CompressedAudio'
        QtMocHelpers::PropertyData<QString>(62, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isDownloadAllowed'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'useSoftwareRenderer'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isBox2DInstalled'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'serverConnectionAccepted'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'serverStatusColor'
        QtMocHelpers::PropertyData<QString>(67, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Platform'
        QtMocHelpers::EnumData<enum Platform>(50, 50, QMC::EnumFlags{}).add({
            {   68, Platform::Linux },
            {   69, Platform::Windows },
            {   70, Platform::MacOSX },
            {   71, Platform::Android },
            {   72, Platform::Ios },
            {   73, Platform::Blackberry },
            {   74, Platform::UbuntuTouchOS },
        }),
    };
    return QtMocHelpers::metaObjectData<ApplicationInfo, qt_meta_tag_ZN15ApplicationInfoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ApplicationInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ApplicationInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ApplicationInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ApplicationInfoE_t>.metaTypes,
    nullptr
} };

void ApplicationInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationInfo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applicationWidthChanged(); break;
        case 1: _t->portraitModeChanged(); break;
        case 2: _t->ratioChanged(); break;
        case 3: _t->fontRatioChanged(); break;
        case 4: _t->applicationSettingsChanged(); break;
        case 5: _t->fullscreenChanged(); break;
        case 6: _t->useSoftwareRendererChanged(); break;
        case 7: _t->isBox2DInstalledChanged(); break;
        case 8: _t->serverConnectionAcceptedChanged(); break;
        case 9: _t->serverStatusColorChanged(); break;
        case 10: { QStringList _r = _t->getResourceDataPaths();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->getAudioFilePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->getAudioFilePathForLocale((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->getLocaleFilePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { QStringList _r = _t->getSystemExcludedFonts();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 15: { QStringList _r = _t->getFontsFromRcc();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 16: { QStringList _r = _t->getBackgroundMusicFromRcc();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->notifyPortraitMode(); break;
        case 19: _t->notifyFullscreenChanged(); break;
        case 20: { QString _r = _t->getVoicesLocale((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->requestAudioFocus();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->abandonAudioFocus(); break;
        case 23: { int _r = _t->localeCompare((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 24: { int _r = _t->localeCompare((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 25: { QVariantList _r = _t->localeSort((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 26: { QVariantList _r = _t->localeSort((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->switchLocale((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->switchLocale(); break;
        case 29: _t->setServerConnectionAccepted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->setServerStatusColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: { Qt::ScreenOrientation _r = _t->getNativeOrientation();
            if (_a[0]) *reinterpret_cast<Qt::ScreenOrientation*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->setRequestedOrientation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: { int _r = _t->getRequestedOrientation();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->sensorIsSupported((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setKeepScreenOn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::applicationWidthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::portraitModeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::ratioChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::fontRatioChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::applicationSettingsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::fullscreenChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::useSoftwareRendererChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::isBox2DInstalledChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::serverConnectionAcceptedChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationInfo::*)()>(_a, &ApplicationInfo::serverStatusColorChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->applicationWidth(); break;
        case 1: *reinterpret_cast<enum Platform*>(_v) = _t->platform(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isMobile(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasShader(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isPortraitMode(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->ratio(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->fontRatio(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->localeShort(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->GCVersion(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->GCVersionCode(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->QTVersion(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->QTMinorVersion(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->OpenSSLVersion(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->CompressedAudio(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isDownloadAllowed(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->useSoftwareRenderer(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isBox2DInstalled(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->m_serverConnectionAccepted; break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->m_serverStatusColor; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setApplicationWidth(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setIsPortraitMode(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setUseSoftwareRenderer(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setServerConnectionAccepted(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setServerStatusColor(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ApplicationInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ApplicationInfoE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ApplicationInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void ApplicationInfo::applicationWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ApplicationInfo::portraitModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ApplicationInfo::ratioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ApplicationInfo::fontRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ApplicationInfo::applicationSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ApplicationInfo::fullscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ApplicationInfo::useSoftwareRendererChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ApplicationInfo::isBox2DInstalledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ApplicationInfo::serverConnectionAcceptedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ApplicationInfo::serverStatusColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
