/****************************************************************************
** Meta object code from reading C++ file 'netconst.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../gcompris/src/core/netconst.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'netconst.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8netconstE_t {};
} // unnamed namespace

template <> constexpr inline auto netconst::qt_create_metaobjectdata<qt_meta_tag_ZN8netconstE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "netconst",
        "MessageType",
        "LOGIN_LIST",
        "LOGIN_REPLY",
        "LOGIN_ACCEPT",
        "DISCONNECT",
        "ACTIVITY_DATA",
        "DATASET_CREATION",
        "DATASET_REMOVE",
        "DATASET_REMOVE_ALL",
        "PING",
        "PONG",
        "ConnectionStatus",
        "NOT_CONNECTED",
        "NOT_LOGGED",
        "BAD_PASSWORD_INPUT",
        "CONNECTED",
        "CONNECTION_LOST",
        "ALREADY_CONNECTED",
        "DISCONNECTED"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MessageType'
        QtMocHelpers::EnumData<MessageType>(1, 1, QMC::EnumFlags{}).add({
            {    2, MessageType::LOGIN_LIST },
            {    3, MessageType::LOGIN_REPLY },
            {    4, MessageType::LOGIN_ACCEPT },
            {    5, MessageType::DISCONNECT },
            {    6, MessageType::ACTIVITY_DATA },
            {    7, MessageType::DATASET_CREATION },
            {    8, MessageType::DATASET_REMOVE },
            {    9, MessageType::DATASET_REMOVE_ALL },
            {   10, MessageType::PING },
            {   11, MessageType::PONG },
        }),
        // enum 'ConnectionStatus'
        QtMocHelpers::EnumData<ConnectionStatus>(12, 12, QMC::EnumFlags{}).add({
            {   13, ConnectionStatus::NOT_CONNECTED },
            {   14, ConnectionStatus::NOT_LOGGED },
            {   15, ConnectionStatus::BAD_PASSWORD_INPUT },
            {   16, ConnectionStatus::CONNECTED },
            {   17, ConnectionStatus::CONNECTION_LOST },
            {   18, ConnectionStatus::ALREADY_CONNECTED },
            {   19, ConnectionStatus::DISCONNECTED },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN8netconstE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN8netconstE =
    netconst::qt_create_metaobjectdata<qt_meta_tag_ZN8netconstE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN8netconstE =
    qt_staticMetaObjectContent_ZN8netconstE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN8netconstE =
    qt_staticMetaObjectContent_ZN8netconstE.relocatingData;

Q_CONSTINIT const QMetaObject netconst::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN8netconstE.stringdata,
    qt_staticMetaObjectStaticContent_ZN8netconstE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN8netconstE.metaTypes,
    nullptr
} };

QT_WARNING_POP
